@@ -9,6 +9,10 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
+	"github.com/shirou/gopsutil/v4/internal/common"
 )
 
 var timesTests = []struct {
@@ -33,14 +37,10 @@ func TestTimesPlan9(t *testing.T) {
 		t.Run(tt.mockedRootFS, func(t *testing.T) {
 			t.Setenv("HOST_ROOT", filepath.Join("testdata/plan9", tt.mockedRootFS))
 			stats, err := Times(false)
-			skipIfNotImplementedErr(t, err)
-			if err != nil {
-				t.Errorf("error %v", err)
-			}
+			common.SkipIfNotImplementedErr(t, err)
+			require.NoError(t, err)
 			eps := cmpopts.EquateApprox(0, 0.00000001)
-			if !cmp.Equal(stats, tt.stats, eps) {
-				t.Errorf("got: %+v\nwant: %+v", stats, tt.stats)
-			}
+			assert.Truef(t, cmp.Equal(stats, tt.stats, eps), "got: %+v\nwant: %+v", stats, tt.stats)
 		})
 	}
 }
