@@ -48,7 +48,7 @@ func mustSanityCheck(fn *Function, reporter io.Writer) {
 	}
 }
 
-func (s *sanity) diagnostic(prefix, format string, args ...any) {
+func (s *sanity) diagnostic(prefix, format string, args ...interface{}) {
 	fmt.Fprintf(s.reporter, "%s: function %s", prefix, s.fn)
 	if s.block != nil {
 		fmt.Fprintf(s.reporter, ", block %s", s.block)
@@ -58,12 +58,12 @@ func (s *sanity) diagnostic(prefix, format string, args ...any) {
 	io.WriteString(s.reporter, "\n")
 }
 
-func (s *sanity) errorf(format string, args ...any) {
+func (s *sanity) errorf(format string, args ...interface{}) {
 	s.insane = true
 	s.diagnostic("Error", format, args...)
 }
 
-func (s *sanity) warnf(format string, args ...any) {
+func (s *sanity) warnf(format string, args ...interface{}) {
 	s.diagnostic("Warning", format, args...)
 }
 
