@@ -50,6 +50,7 @@ import (
 
 const (
 	defaultTestTimeout         = 10 * time.Second
+	defaultTestShortTimeout    = 10 * time.Millisecond
 	stateRecordingBalancerName = "state_recording_balancer"
 )
 
@@ -753,7 +754,7 @@ func (s) TestClientUpdatesParamsAfterGoAway(t *testing.T) {
 	for {
 		time.Sleep(10 * time.Millisecond)
 		cc.mu.RLock()
-		v := cc.mkp.Time
+		v := cc.keepaliveParams.Time
 		cc.mu.RUnlock()
 		if v == 20*time.Second {
 			// Success
@@ -1041,7 +1042,7 @@ func (s) TestUpdateAddresses_NoopIfCalledWithSameAddresses(t *testing.T) {
 	rb := manual.NewBuilderWithScheme("whatever")
 	rb.InitialState(resolver.State{Addresses: addrsList})
 
-	client, err := Dial("whatever:///this-gets-overwritten",
+	client, err := NewClient("whatever:///this-gets-overwritten",
 		WithTransportCredentials(insecure.NewCredentials()),
 		WithResolvers(rb),
 		WithConnectParams(ConnectParams{
